<?php

namespace GiveMollie\API;

use Exception;
use Give\Donations\Models\Donation;
use Give\Framework\PaymentGateways\Exceptions\PaymentGatewayException;
use Mollie\Api\Exceptions\ApiException;
use Mollie\Api\MollieApiClient;

/**
 * @since 2.0.0
 */
class MollieApi
{
    /**
     * @var MollieApiClient $mollieApiClient
     */
    public $mollieApiClient;

    /**
     * @var array $errors
     */
    private $errors = false;

    public function __construct()
    {
        $this->setup();
    }

    /**
     * @since 2.0.0
     */
    public function setup()
    {
        try {
            $this->mollieApiClient = new MollieApiClient();

            $apiKey = $this->getApiKey();

            if ( ! empty($apiKey)) {
                $this->mollieApiClient->setApiKey($apiKey);
            } else {
                throw new Exception(__('NO API key is set', 'give-mollie'));
            }
        } catch (ApiException $e) {
            $this->errors = sprintf(__('API call failed: %s', 'give-mollie'), htmlspecialchars($e->getMessage()));
        } catch (Exception $e) {
            $this->errors = htmlspecialchars($e->getMessage());
        }

        return $this->errors;
    }

    /**
     * @since 2.0.0
     */
    public function getApiKey()
    {
        global $give_mollie;

        return give_get_option("_give_{$give_mollie->gateway_slug}_api_key", '');
    }

    /**
     * @since 2.0.0
     *
     * @throws PaymentGatewayException
     */
    public static function client(): MollieApiClient
    {
        $self = new self();

        if ($self->errors) {
            throw new PaymentGatewayException(__('Mollie client API error: ', 'give-mollie') . $self->errors);
        }

        return $self->mollieApiClient;
    }

    /**
     * @since 2.0.0
     */
    public static function errors()
    {
        $self = new self();

        /**
         * Get the Mollie API errors.
         *
         * @since 2.0.0
         *
         * @param array $errors Error list.
         *
         */
        return apply_filters('givewp_mollie_api_errors', $self->errors);
    }

    /**
     * This meta key is used to store the Mollie customer ID in the DB as it's necessary to perform certain API requests.
     *
     * @since 2.0.0
     */
    public static function getCustomerMetaKey(): string
    {
        $key = '_give_mollie_customer_id';
        if (give_is_test_mode()) {
            $key .= '_test';
        }

        return $key;
    }

    /**
     * The Mollie API requires unique descriptions for subscriptions.
     * The same customer, can't have two subscriptions with the same description.
     *
     * @since 2.0.0
     */
    public static function getPaymentDescription(Donation $donation)
    {
        if ($donation->type->isSingle()) {
            $paymentDescription = sprintf(__('%s - One-Time Donation', 'give-mollie'), $donation->formTitle);
        } else {
            $prettyPeriod = give_recurring_pretty_subscription_frequency($donation->subscription->period->getValue(),
                $donation->subscription->installments, false, $donation->subscription->frequency);

            $paymentDescription = sprintf(__('%1$s - %2$s | Give Subscription #%3$s', 'give-mollie'),
                $donation->formTitle,
                $prettyPeriod, $donation->subscriptionId);
        }

        /**
         * Filter the Mollie payment description.
         *
         * @since 2.0.0
         *
         * @param string $paymentDescription Payment description
         */
        return apply_filters('givewp_mollie_payment_description', $paymentDescription);
    }
}
