<?php

namespace GiveMollie\Gateway;

use Exception;
use Give\Donations\Models\Donation;
use Give\Donations\Models\DonationNote;
use Give\Donations\ValueObjects\DonationStatus;
use Give\Framework\Http\Response\Types\RedirectResponse;
use Give\Framework\PaymentGateways\Commands\RedirectOffsite;
use Give\Framework\PaymentGateways\Exceptions\PaymentGatewayException;
use Give\Framework\PaymentGateways\Log\PaymentGatewayLog;
use Give\Framework\PaymentGateways\PaymentGateway;
use Give\Framework\Support\Facades\Scripts\ScriptAsset;
use GiveMollie\Actions\CreateMolliePayment;
use GiveMollie\Actions\RefundMolliePayment;
use GiveMollie\API\MollieApi;
use GiveMollie\DataTransferObjects\MollieWebhookRequest;
use GiveMollie\Webhooks\MollieWebhookRequestHandler;
use Mollie\Api\Resources\Payment;

/**
 * @since 2.0.0
 */
class MollieGateway extends PaymentGateway
{
    /**
     * @since 2.0.0
     */
    public $secureRouteMethods = [
        'handleSuccessPaymentReturn',
        'handleCanceledPaymentReturn',
    ];

    /**
     * @since 2.0.0
     */
    public $routeMethods = [
        'webhookNotificationsListener',
    ];

    /**
     * @since 2.0.0
     */
    public function getWebhookNotificationsListener(): string
    {
        return $this->routeMethods[0];
    }

    /**
     * @since 2.0.0
     */
    public function getLegacyFormFieldMarkup(int $formId, array $args): string
    {
        if (give_is_setting_enabled(give_get_option('_give_mollie_collect_billing_details'))) {
            ob_start();
            give_default_cc_address_fields($formId);

            return ob_get_clean();
        }

        return sprintf(
            '<div style="text-align: center;"><img src="%s" alt="Mollie" /></div>',
            GIVE_MOLLIE_PLUGIN_URL . 'assets/dist/mollieLogo.svg'
        );
    }

    /**
     * @since 2.0.0
     */
    public function enqueueScript(int $formId)
    {
        $assets = ScriptAsset::get(trailingslashit(GIVE_MOLLIE_PLUGIN_DIR) . '/build/mollieGateway.asset.php');

        wp_enqueue_script(
            self::id(),
            trailingslashit(GIVE_MOLLIE_PLUGIN_URL) . 'build/mollieGateway.js',
            $assets['dependencies'],
            $assets['version'],
            true
        );

        wp_enqueue_style(
            self::id(),
            trailingslashit(GIVE_MOLLIE_PLUGIN_URL) . 'build/mollieGateway.css',
            [],
            $assets['version']
        );
    }


    /**
     * @since 2.0.0
     */
    public static function id(): string
    {
        return 'mollie';
    }

    /**
     * @since 2.0.0
     */
    public function getId(): string
    {
        return self::id();
    }

    /**
     * @since 2.0.0
     */
    public function getName(): string
    {
        return __('Mollie', 'give-mollie');
    }

    /**
     * @since 2.0.0
     */
    public function getPaymentMethodLabel(): string
    {
        return __('Mollie', 'give-mollie');
    }

    /**
     * @since 2.0.0
     *
     * @throws Exception|PaymentGatewayException
     */
    public function createPayment(Donation $donation, $gatewayData): RedirectOffsite
    {
        try {
            $paymentParameters = $this->getPaymentParameters($donation, $gatewayData);
            $molliePayment = $this->createMolliePayment($donation, $paymentParameters);
            $donation->gatewayTransactionId = $molliePayment->id;
            $donation->save();

            return new RedirectOffsite($molliePayment->getCheckoutUrl());
        } catch (Exception $e) {
            $donation->status = DonationStatus::FAILED();
            $donation->save();

            $errorMessage = $e->getMessage();

            DonationNote::create([
                'donationId' => $donation->id,
                'content' => sprintf(esc_html__('Donation failed. Reason: %s', 'give-mollie'), $errorMessage),
            ]);

            throw new PaymentGatewayException($errorMessage);
        }
    }

    /**
     * @since 2.0.0
     *
     * @throws PaymentGatewayException
     */
    protected function createMolliePayment(Donation $donation, array $paymentParameters): Payment
    {
        return (new CreateMolliePayment())($donation, $paymentParameters);
    }

    /**
     * @since 2.0.0
     *
     * @see https://docs.mollie.com/reference/v2/payments-api/create-payment
     */
    public function getPaymentParameters(Donation $donation, $gatewayData): array
    {
        return [
            'amount' => [
                'value' => $donation->amount->formatToDecimal(),
                'currency' => $donation->amount->getCurrency()->getCode(),
            ],
            'description' => MollieApi::getPaymentDescription($donation),
            'redirectUrl' => $this->getPaymentsReturnURL($donation, $gatewayData),
            'cancelUrl' => $this->getPaymentsCancelURL($donation, $gatewayData),
            'webhookUrl' => $this->getPaymentsWebhookUrl($donation), //"localhost", ".dev" are not acceptable.
            //OPTIONAL
            'metadata' => [
                'name' => $donation->donor->name,
                'email' => $donation->donor->email,
                'give_form_id' => $donation->formId,
                'give_form_title' => $donation->formTitle,
                'give_price_id' => $donation->levelId,
                'date' => $donation->createdAt,
                'user_email' => $donation->donor->email,
                'purchase_key' => $donation->purchaseKey,
                'currency' => $donation->amount->getCurrency()->getCode(),
                'user_info' => [
                    'id' => $donation->donor->id,
                    'title' => $donation->donor->prefix,
                    'email' => $donation->donor->email,
                    'first_name' => $donation->donor->firstName,
                    'last_name' => $donation->donor->lastName,
                    'address' => [
                        'country' => $donation->billingAddress->country,
                        'address1' => $donation->billingAddress->address1,
                        'address2' => $donation->billingAddress->address2,
                        'city' => $donation->billingAddress->city,
                        'state' => $donation->billingAddress->state,
                        'zip' => $donation->billingAddress->zip,
                    ],
                ],
                'donation_id' => $donation->id,
            ],
        ];
    }

    /**
     * @since 2.0.0
     */
    protected function handleSuccessPaymentReturn(array $queryParams): RedirectResponse
    {
        // The transaction status will be set through webhooks
        return new RedirectResponse(esc_url_raw($queryParams['givewp-return-url']));
    }

    /**
     * @since 2.0.0
     *
     * @throws Exception
     */
    protected function handleCanceledPaymentReturn(array $queryParams): RedirectResponse
    {
        $donation = Donation::find((int)$queryParams['donation-id']);

        $donation->status = DonationStatus::CANCELLED();
        $donation->save();

        return new RedirectResponse(esc_url_raw($queryParams['givewp-return-url']));
    }

    /**
     * @since 2.0.0
     */
    private function getPaymentsReturnURL(Donation $donation, $gatewayData): string
    {
        $successReturnUrl = $this->generateSecureGatewayRouteUrl(
            'handleSuccessPaymentReturn',
            $donation->id,
            [
                'donation-id' => $donation->id,
                'givewp-return-url' => $gatewayData['successUrl'],
            ]
        );

        /**
         * Filter the success return URL.
         *
         * @since 2.0.0
         *
         * @param string $successReturnUrl The success return URL
         * @param int    $donationId       The donation ID
         */
        return apply_filters('givewp_mollie_success_return_url', $successReturnUrl, $donation->id);
    }

    /**
     * @since 2.0.0
     */
    private function getPaymentsCancelURL(Donation $donation, $gatewayData): string
    {
        $cancelReturnUrl = $this->generateSecureGatewayRouteUrl(
            'handleCanceledPaymentReturn',
            $donation->id,
            [
                'donation-id' => $donation->id,
                'givewp-return-url' => $gatewayData['cancelUrl'],
            ]
        );

        /**
         * Filter the cancel return URL.
         *
         * @since 2.0.0
         *
         * @param string $cancelReturnUrl The cancel return URL
         * @param int    $donationId      The donation ID
         */
        return apply_filters('givewp_mollie_cancel_return_url', $cancelReturnUrl, $donation->id);
    }

    /**
     * @since 2.0.0
     */
    private function getPaymentsWebhookUrl(Donation $donation): string
    {
        $webhookUrl = $this->generateGatewayRouteUrl(
            $this->getWebhookNotificationsListener(),
            [
                'hook_type' => 'payments',
                'payment_id' => $donation->id,
            ]
        );

        /**
         * @since 2.0.0
         *
         * @param string $webhookUrl The webhook URL
         */
        return apply_filters('givewp_mollie_webhook_url', $webhookUrl);
    }

    /**
     * @since 2.0.0
     */
    public function webhookNotificationsListener()
    {
        $this->handleMollieWebhookRequest();
    }

    /**
     * @since 2.0.0
     */
    public function webhookNotificationsLegacyListener()
    {
        if ($this->isLegacyMollieWebhookRequest()) {
            $this->handleMollieWebhookRequest();
        }
    }

    /**
     * @since 2.0.0
     */
    private function isLegacyMollieWebhookRequest(): bool
    {
        return isset($_GET['give-listener']) && 'mollie' === $_GET['give-listener'];
    }

    /**
     * @since 2.0.0
     */
    private function handleMollieWebhookRequest()
    {
        try {
            $mollieWebhookRequest = MollieWebhookRequest::fromArray($_REQUEST);
            give(MollieWebhookRequestHandler::class)($mollieWebhookRequest);
        } catch (Exception $e) {
            esc_html_e('Mollie Webhook Failed.', 'give-mollie');
            PaymentGatewayLog::error(
                'Mollie Webhook Failed. Error: ' . $e->getMessage()
            );
        }

        exit();
    }

    /**
     * @since 2.0.0
     * @throws PaymentGatewayException|Exception
     */
    public function refundDonation(Donation $donation)
    {
        try {
            (new RefundMolliePayment())($donation);
            $donation->status = DonationStatus::REFUNDED();
            $donation->save();

            DonationNote::create([
                'donationId' => $donation->id,
                'content' => sprintf(
                    __('Donation refunded in Mollie for transaction ID: %s', 'give-mollie'),
                    $donation->gatewayTransactionId
                ),
            ]);
        } catch (Exception $e) {
            DonationNote::create([
                'donationId' => $donation->id,
                'content' => sprintf(
                    __(
                        'Error! Donation %s was NOT refunded. Find more details on the error in the logs at Donations > Tools > Logs. To refund the donation, use the Mollie dashboard.',
                        'give-mollie'
                    ),
                    $donation->id
                ),
            ]);

            throw new PaymentGatewayException($e->getMessage());
        }
    }

    /**
     * Links the transaction ID in the donation details using the ID to the Mollie payment details page.
     *
     * @since 2.0.0
     */
    public function linkTransactionId($gatewayTransactionId, $donationId)
    {
        if (intval($gatewayTransactionId) === intval($donationId)) {
            return $gatewayTransactionId;
        }

        if ( ! empty($gatewayTransactionId)) {
            $gatewayTransactionId = '<a href="https://www.mollie.com/dashboard/payments/' . $gatewayTransactionId . '" target="_blank">' . $gatewayTransactionId . '</a>';
        }

        return $gatewayTransactionId;
    }

    /**
     * Displays the Mollie customer ID linked to profile on the donation details screen.
     *
     * @since 2.0.0
     */
    public function linkCustomerId($donationId)
    {
        $donation = Donation::find($donationId);

        if (self::id() !== $donation->gatewayId) {
            return;
        }

        $mollieCustomerId = Give()->donor_meta->get_meta($donation->donorId, MollieApi::getCustomerMetaKey(), true);

        if ( ! empty($mollieCustomerId)) {
            echo '<div class="give-mollie-customer-id give-admin-box-inside"><p><strong>' . __('Mollie Customer ID',
                    'give-mollie') . '</strong>:&nbsp;<a href="https://www.mollie.com/dashboard/customers/' . $mollieCustomerId . '" target="_blank">' . $mollieCustomerId . '</a></p></div>';
        }
    }
}
