<?php

namespace GiveMollie;

use Give\Framework\Exceptions\Primitives\Exception;
use Give\Framework\PaymentGateways\Exceptions\OverflowException;
use Give\Framework\PaymentGateways\PaymentGatewayRegister;
use Give\Framework\PaymentGateways\Webhooks\EventHandlers\DonationAbandoned;
use Give\Framework\PaymentGateways\Webhooks\EventHandlers\DonationCancelled;
use Give\Framework\PaymentGateways\Webhooks\EventHandlers\DonationCompleted;
use Give\Framework\PaymentGateways\Webhooks\EventHandlers\DonationFailed;
use Give\Framework\PaymentGateways\Webhooks\EventHandlers\DonationPending;
use Give\Framework\PaymentGateways\Webhooks\EventHandlers\DonationRefunded;
use Give\Helpers\Hooks;
use Give\ServiceProviders\ServiceProvider;
use GiveMollie\API\MollieApi;
use GiveMollie\FormBuilder\NewFormDefaultBlocks;
use GiveMollie\Gateway\MollieGateway;
use GiveMollie\LegacyListeners\DispatchGiveMollieAfterPaymentCreated;
use GiveMollie\LegacyListeners\DispatchGiveMollieApiErrors;
use GiveMollie\LegacyListeners\DispatchGiveMollieCustomerCreated;
use GiveMollie\LegacyListeners\DispatchGiveMolliePaymentDescription;
use GiveMollie\LegacyListeners\DispatchGiveMollieRedirectUrl;
use GiveMollie\LegacyListeners\DispatchGiveMollieWebhookEndpoint;
use GiveMollie\LegacyListeners\DispatchGiveMollieWebhookHandler;

/**
 * @since 2.0.0
 */
class MollieServiceProvider implements ServiceProvider
{
    /**
     * @since 2.0.0
     */
    public function register()
    {
        give()->singleton(MollieApi::class);
    }

    /**
     * @since 2.0.0
     *
     * @throws Exception|OverflowException
     */
    public function boot()
    {
        add_action(
            'givewp_register_payment_gateway',
            function (PaymentGatewayRegister $registrar) {
                $registrar->registerGateway(MollieGateway::class);
            }
        );

        Hooks::addAction('init', MollieGateway::class, 'webhookNotificationsLegacyListener');
        Hooks::addAction('givewp_form_builder_new_form', NewFormDefaultBlocks::class);

        $this->registerAsyncEventHandlers();
        $this->dispatchLegacyListeners();
        $this->linkDonationDetailsToMollieDashboard();
    }

    /**
     * @since 2.0.0
     */
    private function registerAsyncEventHandlers()
    {
        Hooks::addAction('givewp_mollie_event_donation_completed', DonationCompleted::class);
        Hooks::addAction('givewp_mollie_event_donation_refunded', DonationRefunded::class);
        Hooks::addAction('givewp_mollie_event_donation_cancelled', DonationCancelled::class);
        Hooks::addAction('givewp_mollie_event_donation_failed', DonationFailed::class);
        Hooks::addAction('givewp_mollie_event_donation_abandoned', DonationAbandoned::class);
        Hooks::addAction('givewp_mollie_event_donation_pending', DonationPending::class);
    }

    /**
     * @since 2.0.0
     */
    private function dispatchLegacyListeners()
    {
        // Actions
        Hooks::addAction('givewp_mollie_webhook_request_handler', DispatchGiveMollieWebhookHandler::class);
        Hooks::addAction('givewp_mollie_customer_created', DispatchGiveMollieCustomerCreated::class);
        Hooks::addAction('givewp_mollie_after_payment_created', DispatchGiveMollieAfterPaymentCreated::class);

        // Filters
        Hooks::addFilter('givewp_mollie_api_errors', DispatchGiveMollieApiErrors::class);
        Hooks::addFilter('givewp_mollie_payment_description', DispatchGiveMolliePaymentDescription::class);
        Hooks::addFilter('givewp_mollie_webhook_url', DispatchGiveMollieWebhookEndpoint::class);
        Hooks::addFilter('givewp_mollie_success_return_url', DispatchGiveMollieRedirectUrl::class, '__invoke', 10, 2);
    }

    /**
     * @since 2.0.0
     */
    private function linkDonationDetailsToMollieDashboard()
    {
        Hooks::addFilter('give_payment_details_transaction_id-' . MollieGateway::id(), MollieGateway::class,
            'linkTransactionId', 10, 2);

        Hooks::addFilter('give_view_donation_details_payment_meta_after', MollieGateway::class, 'linkCustomerId');
    }
}
