<?php

namespace GiveMollie\Webhooks;

use Give\Framework\Support\Facades\ActionScheduler\AsBackgroundJobs;
use GiveMollie\DataTransferObjects\MollieWebhookRequest;

/**
 * @since 2.0.0
 */
class MollieWebhookRequestHandler
{
    /**
     * @since 2.0.0
     */
    public function __invoke(MollieWebhookRequest $mollieWebhookRequest)
    {
        /**
         * Allow developers to handle the webhook request.
         *
         * @since 2.0.0
         *
         * @param MollieWebhookRequest $mollieWebhookRequest
         */
        do_action("givewp_mollie_webhook_request_handler", $mollieWebhookRequest);

        if ($this->isRecurringDonation($mollieWebhookRequest)) {
            return;
        }

        if ($mollieWebhookRequest->payment->isPaid() && ! $mollieWebhookRequest->payment->hasRefunds() && ! $mollieWebhookRequest->payment->hasChargebacks()) {
            $this->setCompletedAsyncEvent($mollieWebhookRequest->payment->id);
        } elseif ($mollieWebhookRequest->payment->hasRefunds()) {
            $this->setRefundedAsyncEvent($mollieWebhookRequest->payment->id);
        } elseif ( ! $mollieWebhookRequest->payment->isOpen()) {
            if ($mollieWebhookRequest->payment->isPending()) {
                $this->setPendingAsyncEvent($mollieWebhookRequest->payment->id);
            } elseif ($mollieWebhookRequest->payment->isCanceled()) {
                $this->setCancelledAsyncEvent($mollieWebhookRequest->payment->id);
            } elseif ($mollieWebhookRequest->payment->isFailed()) {
                $this->setFailedAsyncEvent($mollieWebhookRequest->payment->id);
            } else {
                $this->setAbandonedAsyncEvent($mollieWebhookRequest->payment->id); // The payment isn't paid and isn't open anymore. We can assume it was abandoned.
            }
        }
    }

    /**
     * @since 2.0.0
     */
    private function isRecurringDonation(MollieWebhookRequest $mollieWebhookRequest): bool
    {
        return 'subscription' === $mollieWebhookRequest->hookType || $mollieWebhookRequest->payment->hasSequenceTypeFirst();
    }

    /**
     * @since 2.0.0
     */
    private function setCompletedAsyncEvent(string $gatewayTransactionId)
    {
        $this->enqueueAsyncAction('givewp_mollie_event_donation_completed', [$gatewayTransactionId]);
    }

    /**
     * @since 2.0.0
     */
    private function setRefundedAsyncEvent(string $gatewayTransactionId)
    {
        $this->enqueueAsyncAction('givewp_mollie_event_donation_refunded', [$gatewayTransactionId]);
    }

    /**
     * @since 2.0.0
     */
    private function setCancelledAsyncEvent(string $gatewayTransactionId)
    {
        $this->enqueueAsyncAction('givewp_mollie_event_donation_cancelled', [$gatewayTransactionId]);
    }

    /**
     * @since 2.0.0
     */
    private function setFailedAsyncEvent(string $gatewayTransactionId)
    {
        $this->enqueueAsyncAction('givewp_mollie_event_donation_failed', [$gatewayTransactionId]);
    }

    /**
     * @since 2.0.0
     */
    private function setAbandonedAsyncEvent(string $gatewayTransactionId)
    {
        $this->enqueueAsyncAction('givewp_mollie_event_donation_abandoned', [$gatewayTransactionId]);
    }

    /**
     * @since 2.0.0
     */
    private function setPendingAsyncEvent(string $gatewayTransactionId)
    {
        $this->enqueueAsyncAction('givewp_mollie_event_donation_pending', [$gatewayTransactionId]);
    }

    /**
     * @since 2.0.0
     */
    private function enqueueAsyncAction(string $hook, array $args)
    {
        AsBackgroundJobs::enqueueAsyncAction($hook, $args, 'give-mollie');
    }
}
